@@ -9,6 +9,7 @@ from snappi_ixnetwork.ngpf import Ngpf
 from snappi_ixnetwork.trafficitem import TrafficItem
 from snappi_ixnetwork.capture import Capture
 from snappi_ixnetwork.timer import Timer
+from snappi_ixnetwork.protocolmetrics import ProtocolMetrics
 
 
 class Api(snappi.Api):
@@ -50,11 +51,17 @@ class Api(snappi.Api):
         self._ixn_errors = list()
         self._config_objects = {}
         self._device_encap = {}
+        self._config_type = self.config()
+        self._transmit_state = self.transmit_state()
+        self._link_state = self.link_state()
+        self._capture_state = self.capture_state()
+        self._capture_request = self.capture_request()
         self.validation = Validation(self)
         self.vport = Vport(self)
         self.ngpf = Ngpf(self)
         self.traffic_item = TrafficItem(self)
         self.capture = Capture(self)
+        self.protocol_metrics = ProtocolMetrics(self)
 
     def _get_addr_port(self, host):
         items = host.split('/')
@@ -74,10 +81,18 @@ class Api(snappi.Api):
         return self._config
 
     def get_config_object(self, name):
-        return self._config_objects[name]
+        try:
+            return self._config_objects[name]
+        except KeyError:
+            raise NameError("snappi object named {0} not found"
+                            .format(name))
 
     def get_device_encap(self, name):
-        return self._device_encap[name]
+        try:
+            return self._device_encap[name]
+        except KeyError:
+            raise NameError("snappi object named {0} not found"
+                            .format(name))
 
     @property
     def ixn_objects(self):
@@ -136,16 +151,22 @@ class Api(snappi.Api):
     def set_config(self, config):
         """Set or update the configuration
         """
+        if isinstance(config, (type(self._config_type),
+                                str)) is False:
+            raise TypeError(
+                'The content must be of type Union[Config, str]')
+        
         if isinstance(config, str) is True:
-            config = self.config().deserialize(config)
-        self._config = config
+            config = self._config_type.deserialize(config)
         self._config_objects = {}
         self._device_encap = {}
         self._ixn_objects = {}
         self._connect()
+        self._config = self._validate_instance(
+                            config)
         with Timer(self, 'Config validation'):
             self.validation.validate_config()
-        if self._config is None:
+        if len(self._config._properties) == 0:
             self._ixnetwork.NewConfig()
         else:
             self.vport.config()
@@ -154,44 +175,60 @@ class Api(snappi.Api):
             with Timer(self, 'Flows configuration'):
                 self.traffic_item.config()
         self._running_config = self._config
+        self._apply_change()
         return self._request_detail()
 
-    def set_transmit_state(self, flow_transmit_state):
+    def set_transmit_state(self, payload):
         """Set the transmit state of flows
         """
-        if isinstance(flow_transmit_state, str) is True:
-            flow_transmit_state = self.transmit_state().deserialize(
-                                    flow_transmit_state)
+        if isinstance(payload, (type(self._transmit_state),
+                                str)) is False:
+            raise TypeError(
+                'The content must be of type Union[TransmitState, str]')
+        if isinstance(payload, str) is True:
+            payload = self._transmit_state.deserialize(
+                                    payload)
         self._connect()
-        self.traffic_item.transmit(flow_transmit_state)
+        self.traffic_item.transmit(payload)
         return self._request_detail()
 
     def set_link_state(self, link_state):
+        if isinstance(link_state, (type(self._link_state),
+                                str)) is False:
+            raise TypeError(
+                'The content must be of type Union[LinkState, str]')
         if isinstance(link_state, str):
-            link_state = self.link_state().deserialize(link_state)
+            link_state = self._link_state.deserialize(link_state)
         self._connect()
         if link_state.port_names is not None:
             self.vport.set_link_state(link_state)
         return self._request_detail()
     
-    def set_capture_state(self, request):
+    def set_capture_state(self, payload):
         """Starts capture on all ports that have capture enabled.
         """
+        if isinstance(payload, (type(self._capture_state),
+                                str)) is False:
+            raise TypeError(
+                'The content must be of type Union[CaptureState, str]')
+        if isinstance(payload, str) is True:
+            payload = self._capture_state.deserialize(
+                                    payload)
         self._connect()
-        self.capture.set_capture_state(request)
+        self.capture.set_capture_state(payload)
         return self._request_detail()
 
     def get_capture(self, request):
         """Gets capture file and returns it as a byte stream
         """
-        self._connect()
-        if isinstance(request, (type(self.capture_request()),
+        if isinstance(request, (type(self._capture_request),
                                 str)) is False:
             raise TypeError(
                 'The content must be of type Union[CaptureRequest, str]')
         if isinstance(request, str) is True:
-            request = self.capture_request().deserialize(
+            request = self._capture_request.deserialize(
                 request)
+        self._connect()
         return self.capture.results(request)
     
     def get_metrics(self, request):
@@ -225,6 +262,13 @@ class Api(snappi.Api):
             metric_res = self.metrics_response()
             metric_res.flow_metrics.deserialize(response)
             return metric_res
+        if request.choice in self.protocol_metrics.get_supported_protocols():
+            response = self.protocol_metrics.results(request)
+            metric_res = self.metrics_response()
+            getattr(
+                metric_res, request.choice + '_metrics'
+            ).deserialize(response)
+            return metric_res
         if request.choice is not None:
             msg = "{} is not a supported choice for metrics; \
             the supported choices are \
@@ -281,6 +325,38 @@ class Api(snappi.Api):
         if len(app_errors) > 0:
             self._ixn_errors = app_errors[0].Error.find()
 
+    def _validate_instance(self, config):
+        """Validate current IxNetwork instance:
+        1. Stop everything if local config is None
+        2. Otherwise add warning message """
+        traffic_state = self._traffic.State
+        if self.snappi_config is None:
+            if traffic_state == 'started':
+                self._traffic_item.find()
+                if len(self._traffic_item) > 0:
+                    self._traffic_item.StopStatelessTrafficBlocking()
+            if self._globals.Topology.Status  == 'started':
+                self._ixnetwork.StopAllProtocols('sync')
+        else:
+            if traffic_state == 'started':
+                msg = "Flows are in running state. " \
+                      "Please stop those using set_transmit_state"
+                self.add_error(msg)
+                self.warning(msg)
+        return config
+
+    def _apply_change(self):
+        """Apply on the fly only applicable for Device object"""
+        glob_topo = self._globals.Topology
+        if glob_topo.ApplyOnTheFlyState == 'allowed':
+            url = '%s/globals/topology/operations/applyonthefly' % self._ixnetwork.href
+            payload = {'arg1': glob_topo.href}
+            # todo: Sometime it redirect to some unknown loaction
+            try:
+                self._request('POST', url, payload)
+            except Exception:
+                pass
+        
     def _request(self, method, url, payload=None):
         connection, url = self._assistant.Session._connection._normalize_url(
             url)
