@@ -1,6 +1,5 @@
 from snappi_ixnetwork.timer import Timer
 from snappi_ixnetwork.customfield import CustomField
-import snappi
 
 
 class TrafficItem(CustomField):
@@ -235,12 +234,7 @@ class TrafficItem(CustomField):
         else:
             encap = None
             for name in flow.tx_rx.device.tx_names:
-                device = self._api.get_config_object(name)
-                if device.__class__ \
-                        is snappi.DeviceBgpv4RouteRange:
-                    encap = 'ipv4'
-                else:
-                    encap = self._api.get_device_encap(name)
+                encap = self._api.get_device_encap(name)
         return encap
 
     def _configure_endpoint(self, ixn_endpoint_set, endpoint):
@@ -445,7 +439,7 @@ class TrafficItem(CustomField):
             args['Type'] = "fixed"
             args['FixedSize'] = size.fixed
         elif size.choice == 'increment':
-            args['Type'] = "incrment"
+            args['Type'] = "increment"
             args['IncrementFrom'] = size.increment.start
             args['IncrementTo'] = size.increment.end
             args['IncrementStep'] = size.increment.step
@@ -473,11 +467,10 @@ class TrafficItem(CustomField):
             args['Type'] = 'framesPerSecond'
             value = rate.pps
         else:
-            # TODO: fix for other units
             args['Type'] = 'bitsPerSecond'
             args['BitRateUnitsType'] = TrafficItem._BIT_RATE_UNITS_TYPE[
-                rate.unit]
-            value = rate.bps
+                rate.choice]
+            value = getattr(rate, rate.choice)
         args['Rate'] = value
         self._update(ixn_frame_rate, **args)
 
